@@ -1,30 +1,43 @@
 from __future__ import absolute_import
-# Copyright (c) 2010-2017 openpyxl
+# Copyright (c) 2010-2018 openpyxl
 
 """Workbook is the top-level container for all document information."""
+from copy import copy
 
-from openpyxl.compat import deprecated
-from openpyxl.compat import OrderedDict
+from openpyxl.compat import deprecated, long
 from openpyxl.worksheet import Worksheet
 from openpyxl.worksheet.read_only import ReadOnlyWorksheet
+from openpyxl.worksheet.write_only import WriteOnlyWorksheet
 from openpyxl.worksheet.copier import WorksheetCopy
 
+from openpyxl.utils import quote_sheetname
 from openpyxl.utils.indexed_list import IndexedList
 from openpyxl.utils.datetime  import CALENDAR_WINDOWS_1900
 from openpyxl.utils.exceptions import ReadOnlyWorkbookException
 
-from openpyxl.writer.write_only import WriteOnlyWorksheet, save_dump
-from openpyxl.writer.excel import save_workbook
+from openpyxl.writer.excel import save_workbook, save_dump
 
 from openpyxl.styles.cell_style import StyleArray
 from openpyxl.styles.named_styles import NamedStyle
 from openpyxl.styles.differential import DifferentialStyleList
+from openpyxl.styles.alignment import Alignment
+from openpyxl.styles.borders import DEFAULT_BORDER
+from openpyxl.styles.fills import DEFAULT_EMPTY_FILL, DEFAULT_GRAY_FILL
+from openpyxl.styles.fonts import DEFAULT_FONT
+from openpyxl.styles.protection import Protection
+from openpyxl.styles.colors import COLOR_INDEX
+from openpyxl.styles.named_styles import NamedStyleList
+from openpyxl.styles.table import TableStyleList
 
 from openpyxl.chartsheet import Chartsheet
 from .defined_name import DefinedName, DefinedNameList
 from openpyxl.packaging.core import DocumentProperties
 from openpyxl.packaging.relationship import RelationshipList
+from .child import _WorkbookChild
 from .protection import DocumentSecurity
+from .properties import CalcProperties
+from .views import BookView
+
 
 from openpyxl.xml.constants import (
     XLSM,
@@ -33,6 +46,7 @@ from openpyxl.xml.constants import (
     XLTX
 )
 
+INTEGER_TYPES = (int, long)
 
 class Workbook(object):
     """Workbook is the container for all other parts of the document."""
@@ -45,8 +59,10 @@ class Workbook(object):
 
     def __init__(self,
                  write_only=False,
+                 iso_dates=False,
                  ):
         self._sheets = []
+        self._pivots = []
         self._active_sheet_index = 0
         self.defined_names = DefinedNameList()
         self._external_links = []
@@ -62,24 +78,20 @@ class Workbook(object):
         self.is_template = False
         self._differential_styles = DifferentialStyleList()
         self.code_name = None
-        self.excel_base_date = CALENDAR_WINDOWS_1900
+        self.epoch = CALENDAR_WINDOWS_1900
         self.encoding = "utf-8"
+        self.iso_dates = iso_dates
 
         if not self.write_only:
             self._sheets.append(Worksheet(self))
 
         self.rels = RelationshipList()
+        self.calculation = CalcProperties()
+        self.views = [BookView()]
 
 
     def _setup_styles(self):
         """Bootstrap styles"""
-        from openpyxl.styles.alignment import Alignment
-        from openpyxl.styles.borders import DEFAULT_BORDER
-        from openpyxl.styles.fills import DEFAULT_EMPTY_FILL, DEFAULT_GRAY_FILL
-        from openpyxl.styles.fonts import DEFAULT_FONT
-        from openpyxl.styles.protection import Protection
-        from openpyxl.styles.colors import COLOR_INDEX
-        from openpyxl.styles.named_styles import NamedStyleList
 
         self._fonts = IndexedList()
         self._fonts.add(DEFAULT_FONT)
@@ -100,7 +112,8 @@ class Workbook(object):
         self._colors = COLOR_INDEX
         self._cell_styles = IndexedList([StyleArray()])
         self._named_styles = NamedStyleList()
-        self.add_named_style(NamedStyle(font=DEFAULT_FONT, builtinId=0))
+        self.add_named_style(NamedStyle(font=copy(DEFAULT_FONT), builtinId=0))
+        self._table_styles = TableStyleList()
 
 
     @property
@@ -124,9 +137,17 @@ class Workbook(object):
         """Returns the current active sheet."""
         return self.active
 
+
+    @property
+    def excel_base_date(self):
+        return self.epoch
+
     @property
     def active(self):
-        """Get the currently active sheet or None"""
+        """Get the currently active sheet or None
+
+        :type: :class:`openpyxl.worksheet.worksheet.Worksheet`
+        """
         try:
             return self._sheets[self._active_sheet_index]
         except IndexError:
@@ -135,13 +156,29 @@ class Workbook(object):
     @active.setter
     def active(self, value):
         """Set the active sheet"""
-        self._active_sheet_index = value
+        if not isinstance(value, (_WorkbookChild, INTEGER_TYPES)):
+            raise TypeError("Value must be either a worksheet, chartsheet or numerical index")
+        if isinstance(value, INTEGER_TYPES):
+            self._active_sheet_index = value
+            return
+            #if self._sheets and 0 <= value < len(self._sheets):
+                #value = self._sheets[value]
+            #else:
+                #raise ValueError("Sheet index is outside the range of possible values", value)
+        if value not in self._sheets:
+            raise ValueError("Worksheet is not in the workbook")
+        if value.sheet_state != "visible":
+            raise ValueError("Only visible sheets can be made active")
+
+        idx = self._sheets.index(value)
+        self._active_sheet_index = idx
+
 
     def create_sheet(self, title=None, index=None):
         """Create a worksheet (at an optional index).
 
         :param title: optional title of the sheet
-        :type tile: unicode
+        :type title: unicode
         :param index: optional position at which the sheet will be inserted
         :type index: int
 
@@ -174,13 +211,17 @@ class Workbook(object):
 
 
     def remove(self, worksheet):
-        """Remove a worksheet from this workbook."""
+        """Remove `worksheet` from this workbook."""
+        idx = self._sheets.index(worksheet)
+        localnames = self.defined_names.localnames(scope=idx)
+        for name in localnames:
+            self.defined_names.delete(name, scope=idx)
         self._sheets.remove(worksheet)
 
 
     @deprecated("Use wb.remove(worksheet) or del wb[sheetname]")
     def remove_sheet(self, worksheet):
-        """Remove a worksheet from this workbook."""
+        """Remove `worksheet` from this workbook."""
         self.remove(worksheet)
 
 
@@ -224,7 +265,7 @@ class Workbook(object):
         :type name: string
 
         """
-        for sheet in self.worksheets:
+        for sheet in self.worksheets + self.chartsheets:
             if sheet.title == key:
                 return sheet
         raise KeyError("Worksheet {0} does not exist.".format(key))
@@ -243,19 +284,27 @@ class Workbook(object):
 
     @property
     def worksheets(self):
+        """A list of sheets in this workbook
+
+        :type: list of :class:`openpyxl.worksheet.worksheet.Worksheet`
+        """
         return [s for s in self._sheets if isinstance(s, (Worksheet, ReadOnlyWorksheet, WriteOnlyWorksheet))]
 
     @property
     def chartsheets(self):
+        """A list of Chartsheets in this workbook
+
+        :type: list of :class:`openpyxl.chartsheet.chartsheet.Chartsheet`
+        """
         return [s for s in self._sheets if isinstance(s, Chartsheet)]
 
     @property
     def sheetnames(self):
-        """Returns the list of the names of worksheets in the workbook.
+        """Returns the list of the names of worksheets in this workbook.
 
         Names are returned in the worksheets order.
 
-        :rtype: list of strings
+        :type: list of strings
 
         """
         return [s.title for s in self._sheets]
@@ -264,7 +313,7 @@ class Workbook(object):
         """Create a new named_range on a worksheet"""
         defn = DefinedName(name=name, localSheetId=scope)
         if worksheet is not None:
-            defn.value = "{0}!{1}".format(worksheet.title, value)
+            defn.value = "{0}!{1}".format(quote_sheetname(worksheet.title), value)
         else:
             defn.value = value
 
@@ -352,8 +401,10 @@ class Workbook(object):
 
     def copy_worksheet(self, from_worksheet):
         """Copy an existing worksheet in the current workbook
-        :warning: This function cannot copy worksheets between workbooks.
-        worksheets can only be copied within the workbook that they belong
+
+        .. warning::
+            This function cannot copy worksheets between workbooks.
+            worksheets can only be copied within the workbook that they belong
 
         :param from_worksheet: the worksheet to be copied from
         :return: copy of the initial worksheet
@@ -361,7 +412,7 @@ class Workbook(object):
         if self.__write_only or self._read_only:
             raise ValueError("Cannot copy worksheets in read-only or write-only mode")
 
-        new_title = "{0} Copy".format(from_worksheet.title)
+        new_title = u"{0} Copy".format(from_worksheet.title)
         to_worksheet = self.create_sheet(title=new_title)
         cp = WorksheetCopy(source_worksheet=from_worksheet, target_worksheet=to_worksheet)
         cp.copy_worksheet()
