@@ -6,6 +6,7 @@ from copy import copy
 from openpyxl.compat.strings import safe_repr
 from openpyxl.descriptors import Strict
 from openpyxl.descriptors import MinMax, Sequence
+from openpyxl.descriptors.serialisable import Serialisable
 
 from openpyxl.utils import (
     range_boundaries,
@@ -15,7 +16,7 @@ from openpyxl.utils import (
 )
 
 
-class CellRange(Strict):
+class CellRange(Serialisable):
     """
     Represents a range in a sheet: title and coordinates.
 
@@ -78,7 +79,7 @@ class CellRange(Strict):
     @property
     def coord(self):
         """
-        Excel style representation of the range
+        Excel-style representation of the range
         """
         fmt = "{min_col}{min_row}:{max_col}{max_row}"
         if (self.min_col == self.max_col
@@ -116,9 +117,9 @@ class CellRange(Strict):
     def _get_range_string(self):
         fmt = "{coord}"
         title = self.title
-        if self.title:
+        if title:
             fmt = u"{title}!{coord}"
-            title = quote_sheetname(self.title)
+            title = quote_sheetname(title)
         return fmt.format(title=title, coord=self.coord)
 
     __unicode__ = _get_range_string
@@ -159,7 +160,7 @@ class CellRange(Strict):
         """
         Test whether the ranges are not equal.
 
-        :type other: CellRange
+        :type other: openpyxl.worksheet.cell_range.CellRange
         :param other: Other sheet range
         :return: ``True`` if *range* != *other*.
         """
@@ -173,14 +174,14 @@ class CellRange(Strict):
             or self.max_row != other.max_row
             or other.min_col != self.min_col
             or self.max_col != other.max_col
-                )
+        )
 
 
     def __eq__(self, other):
         """
         Test whether the ranges are equal.
 
-        :type other: CellRange
+        :type other: openpyxl.worksheet.cell_range.CellRange
         :param other: Other sheet range
         :return: ``True`` if *range* == *other*.
         """
@@ -189,9 +190,9 @@ class CellRange(Strict):
 
     def issubset(self, other):
         """
-        Test whether every element in the range is in *other*.
+        Test whether every cell in this range is also in *other*.
 
-        :type other: SheetRange
+        :type other: openpyxl.worksheet.cell_range.CellRange
         :param other: Other sheet range
         :return: ``True`` if *range* <= *other*.
         """
@@ -208,7 +209,7 @@ class CellRange(Strict):
 
     def __lt__(self, other):
         """
-        Test whether every element in the range is in *other*, but not all.
+        Test whether *other* contains every cell of this range, and more.
 
         :type other: openpyxl.worksheet.cell_range.CellRange
         :param other: Other sheet range
@@ -219,10 +220,10 @@ class CellRange(Strict):
 
     def issuperset(self, other):
         """
-        Test whether every element in *other* is in the range.
+        Test whether every cell in *other* is in this range.
 
-        :type other: openpyxl.worksheet.cell_range.CellRange or tuple[int, int]
-        :param other: Other sheet range or cell index (*row_idx*, *col_idx*).
+        :type other: openpyxl.worksheet.cell_range.CellRange
+        :param other: Other sheet range
         :return: ``True`` if *range* >= *other* (or *other* in *range*).
         """
         self._check_title(other)
@@ -233,7 +234,6 @@ class CellRange(Strict):
             (self.min_col <= other.min_col <= other.max_col <= self.max_col)
         )
 
-
     __ge__ = issuperset
 
 
@@ -249,7 +249,7 @@ class CellRange(Strict):
 
     def __gt__(self, other):
         """
-        Test whether every element in *other* is in the range, but not all.
+        Test whether this range contains every cell in *other*, and more.
 
         :type other: openpyxl.worksheet.cell_range.CellRange
         :param other: Other sheet range
@@ -260,36 +260,36 @@ class CellRange(Strict):
 
     def isdisjoint(self, other):
         """
-        Return ``True`` if the range has no elements in common with other.
+        Return ``True`` if this range has no cell in common with *other*.
         Ranges are disjoint if and only if their intersection is the empty range.
 
         :type other: openpyxl.worksheet.cell_range.CellRange
         :param other: Other sheet range.
-        :return: `True`` if the range has no elements in common with other.
+        :return: ``True`` if the range has no cells in common with other.
         """
         self._check_title(other)
 
-        # sort by top-left vertex
+        # Sort by top-left vertex
         if self.bounds > other.bounds:
-            i = self
-            self = other
-            other = i
+            self, other = other, self
 
-        return (self.max_col, self.max_row) < (other.min_col, other.max_row)
+        return (self.max_col < other.min_col
+                or self.max_row < other.min_row
+                or other.max_row < self.min_row)
 
 
     def intersection(self, other):
         """
-        Return a new range with elements common to the range and another
+        Return a new range with cells common to this range and *other*
 
-        :type others: tuple[openpyxl.worksheet.cell_range.CellRange]
-        :param others: Other sheet ranges.
-        :return: the current sheet range.
-        :raise: :class:`ValueError` if an *other* range don't intersect
-            with the current range.
+        :type other: openpyxl.worksheet.cell_range.CellRange
+        :param other: Other sheet range.
+        :return: the intersecting sheet range.
+        :raise: :class:`ValueError` if the *other* range doesn't intersect
+            with this range.
         """
         if self.isdisjoint(other):
-            raise ValueError("Range {0} don't intersect {0}".format(self, other))
+            raise ValueError("Range {0} doesn't intersect {0}".format(self, other))
 
         min_row = max(self.min_row, other.min_row)
         max_row = min(self.max_row, other.max_row)
@@ -304,11 +304,13 @@ class CellRange(Strict):
 
     def union(self, other):
         """
-        Return a new range with elements from the range and all *others*.
+        Return the minimal superset of this range and *other*. This new range
+        will contain all cells from this range, *other*, and any additional
+        cells required to form a rectangular ``CellRange``.
 
-        :type others: tuple[openpyxl.worksheet.cell_range.CellRange]
-        :param others: Other sheet ranges.
-        :return: the current sheet range.
+        :type other: openpyxl.worksheet.cell_range.CellRange
+        :param other: Other sheet range.
+        :return: a ``CellRange`` that is a superset of this and *other*.
         """
         self._check_title(other)
 
@@ -319,7 +321,6 @@ class CellRange(Strict):
         return CellRange(min_col=min_col, min_row=min_row, max_col=max_col,
                          max_row=max_row, title=self.title)
 
-
     __or__ = union
 
 
@@ -363,7 +364,7 @@ class CellRange(Strict):
 
     @property
     def size(self):
-        """ Return the size of the range as a dicitionary of rows and columns. """
+        """ Return the size of the range as a dictionary of rows and columns. """
         cols = self.max_col + 1 - self.min_col
         rows = self.max_row + 1 - self.min_row
         return {'columns':cols, 'rows':rows}
@@ -402,16 +403,23 @@ class MultiCellRange(Strict):
 
     def add(self, coord):
         """
-        Add a cell coordinate. Will create a new CellRange
+        Add a cell coordinate or CellRange
         """
+        cr = None
+        if isinstance(coord, CellRange):
+            cr = coord
+            coord = cr.coord
         if coord not in self:
-            cr = CellRange(coord)
+            if cr is None:
+                cr = CellRange(coord)
             ranges = self.ranges
             ranges.append(cr)
             self.ranges = ranges
-        return self
 
-    __iadd__ = add
+
+    def __iadd__(self, coord):
+        self.add(coord)
+        return self
 
 
     def __eq__(self, other):
